# Copyright 2012 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import sys

from mocker import MockerTestCase

from offspring.networking import (
    TimeoutHTTP,
    TimeoutHTTPConnection,
    TimeoutTransport,
)

class TestTimeoutTransport(MockerTestCase):

    def test_make_connection_before_python2_6(self):
        """
        When running on versions earlier than python 2.7, TimeoutTransport's
        make_connection method should return an instance of TimeoutHTTP as
        before python 2.7 xmlrpclib continued to use python 1.5 httplib API.
        """
        mock_sys = self.mocker.replace(sys)
        mock_sys.version_info
        self.mocker.result((2,6))
        self.mocker.replay()
        transport = TimeoutTransport()
        connection = transport.make_connection("http://localhost:1234")
        self.assertTrue(isinstance(connection, TimeoutHTTP))

    def test_make_connection_on_python2_7_or_later(self):
        """
        When running on versions python 2.7 or later, TimeoutTransport's
        make_connection method should return an instance of 
        TimeoutHTTPConnection instead of TimeoutHTTP.
        """
        mock_sys = self.mocker.replace(sys)
        mock_sys.version_info
        self.mocker.result((2,7))
        self.mocker.replay()
        transport = TimeoutTransport()
        connection = transport.make_connection("http://localhost:1234")
        self.assertTrue(isinstance(connection, TimeoutHTTPConnection))

    def test_make_connection_saves_extra_headers(self):
        """
        TimeoutTransport's make_connection should save extra headers returned
        when it calls it's get_host_info method as extra_headers. It'll be set
        to a list if there is auth details in host url.
        
        After python 2.6, the authorization header will not be sent if it
        hasn't been saved to extra_headers when make_connection is called.
        """
        transport = TimeoutTransport()
        transport.make_connection("http://admin:password@localhost:1234")
        self.assertTrue(transport._extra_headers is not None)
